% Merge multiple clusters into "num_clusters" number of clusters
%
function Lbls = mergeClusters(X, Lbls, num_clusters, merge_method, mfaParams, mfaCenterLbls, assign_bsv)
    % this is the matlab merging version
    %Lbls1 = mergeClustersMatlab(X, Lbls, num_clusters, merge_method, mfaParams, mfaCenterLbls, assign_bsv);
   
    % and this is the java one
    merger = manifolds.MergeClusters;
    means = mfaParams{2};
    Lbls = merger.mergeClusters(X, Lbls, num_clusters, merge_method, means, mfaCenterLbls, assign_bsv); 
    
function Lbls = mergeClustersMatlab(X, Lbls, num_clusters, merge_method, mfaParams, mfaCenterLbls, assign_bsv)    
    uniqueLbls = unique(Lbls);
    num_lbls = size(uniqueLbls,1);
    if (find(uniqueLbls == 0))
        num_lbls = num_lbls - 1;
        uniqueLbls = uniqueLbls(2:size(uniqueLbls,1));
    end;
    if (num_lbls <= num_clusters)
        return;
    end;
    
    % merge two clusters at a time until there are just num_clusters clusters
    % overall
    means = mfaParams{2};
    while (num_lbls > num_clusters)
        if (merge_method == 0)  % merge according to the closest pair of each two clusters
            [c1, c2] = closestClusters(X, Lbls);
        else 
            % merge according to the distance of every pair of centers of mfa components
            uniquemfa_lbls = unique(mfaCenterLbls);
            nummfa_clust = size(uniquemfa_lbls);
            if (find(uniquemfa_lbls == 0))
                nummfa_clust = nummfa_clust-1;
            end;
            if (nummfa_clust > num_clusters)
                [c1, c2] = closestClusters(means, mfaCenterLbls); 
                inds = find(mfaCenterLbls == c2);
                mfaCenterLbls(inds) = c1;
            else 
                [c1, c2] = closestClusters(X, Lbls);               
            end;
        end;
        inds = find(Lbls == c2);
        Lbls(inds) = c1;
        num_lbls = num_lbls -1;
    end;
    MergedClusters = Lbls
    L = Lbls(find(Lbls ~= 0));
    uniqueLbls = unique(L);
    for i = 1:size(uniqueLbls,1)
        old_lbl = uniqueLbls(i,1);
        inds = find(Lbls == old_lbl);
        Lbls(inds,1) = i;
    end;
    if (assign_bsv)
        Lbls = assignBSVectors(Lbls, X);
    end;
    
function L = assignBSVectors(Lbls, X)
    L = Lbls;
    for i = 1:size(X,1)
        if (Lbls(i,1) == 0) % this is bounded support vector        
            minInd = 0;
            minDist = realmax;
            for j = 1:size(X,1)
                if (j ~= i && L(j,1) ~= 0)
                    tmpdist = sum((X(i,:) - X(j,:)).^2,2);
                    if (tmpdist < minDist)
                        minDist = tmpdist;
                        minInd = j;
                    end;
                end;
            end;
            L(i,1) = L(minInd,1);            
        end;
    end;
    
function [c1, c2] = closestClusters(X, Lbls)
    unique_lbls = unique(Lbls);
    num_lbls = size(unique_lbls);
    if (find(Lbls == 0)) % there is a bounded support vector
        start = 2;
    else
        start = 1;
    end;
    c1 = 1; c2 = 1;
    mindist = realmax;
    for i = start:num_lbls-1
        inds1 = find(Lbls == unique_lbls(i));        
        for j = i + 1:num_lbls
            inds2 = find(Lbls == unique_lbls(j));
            tmpdist = minPairwiseDist(X(inds1,:), X(inds2,:));
            if (tmpdist < mindist)
                c1 = unique_lbls(i);
                c2 = unique_lbls(j);
                mindist = tmpdist;
            end;
        end;
    end;
    
function D = minPairwiseDist(X, Y)    
    D = realmax;
    for i = 1:size(X,1)
        rX = repmat(X(i,:), size(Y,1),1);
        dists = (rX-Y).*(rX-Y);
        dists = sum(dists,2);
        tmpdist = min(dists);
        if (tmpdist < D)
            D = tmpdist;
        end;
    end;
