function Params = runLibSvm(X, nu, gamma)
    Lbl = zeros(size(X,1),1);
    params1 = ['-s 2 -t 2 -g 10 -n 0.05'];
    %params1 = ['-s 2 -t 2 -g 10 -n 0.1'];
    %params2 = ['-s 2 -t 1 -d 3 -r 10 -n 0.1'];
    params = ['-s 2 -t 2 -g ' num2str(gamma) ' -n ' num2str(nu)];
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % running one class svm and clustering the dense regions, 
    % i.e. running SVC
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    wghts = zeros(size(X,1),1);
    model = svmtrain(Lbl, X, params)
    tf = issparse(model.Parameters)
    tf = issparse(model.sv_coef) 
    tf = issparse(model.SVs) 
    size(model.SVs)
    model.SVs
    %[train_label, accuracy, prob_estimates] = svmpredict(ones(size(X,1),1), X, model);
    train_label = svmpredict1(X, model, gamma);
    svs = full(model.SVs);
    sv_inds = getIndices(X, svs); 
    plotContours(X, model, gamma)
    %Lbls = clusterData(X, model, sv_inds, train_label);        
    Lbls = ones(size(X,1),1);        
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % plot the detected clusters and the (bounded) support vectors
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    Params = cell(2,1);
    Params{1} = model;
    Params{2} = train_label;
    Params{3} = Lbls;
    %plotClustering(X, model, train_label, Lbls);
    %plotContours(X, model, gamma);
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% mimics the svmpredict from the dll but rather than -1 1 returns real 
% value labels corresponding to the margine
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function L = svmpredict1(X, model, gamma) 
    m = size(X,1);
    L = zeros(m,1);
    ro = model.rho;
    for i = 1:m
        L(i,1) = model.sv_coef'*KernelG(X(i,:), model, gamma) - ro;    
    end;
    inds = find((L > -0.0000000001) & (L < 0.0000000001))
    L(inds, 1) = 0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% computes the vector K(x, x_i) i = 1,N
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function K_xxi = KernelG(ptX, model, gamma)
    SVs = model.SVs;
    m = size(SVs,1);
    Xi = repmat(ptX, m, 1);
    normXX = sum((Xi - SVs).*(Xi-SVs),2);
    K = exp(ones(m,1));
    K_xxi = K.^(-gamma*normXX);    
    
function L = clusterData(X, model, sv_inds, train_label)
    A = buildAdjacency(X, model, sv_inds, train_label);
    %A = buildAdjacency1(X, model, sv_inds, train_label);    
    connComp = manifolds.ConnectedComponents;
    L = connComp.clabel(A);

% check the line between each example and the support vectors. 
% If for all the points on the line the radius is smaller than the radius of the
% bounding SV sphere then the point is in one cluster with this support vector
function A = buildAdjacency(X, model, sv_inds, train_label)    
    m = size(X,1);
    n = size(sv_inds,1);
    A = zeros(m);
    for i = 1:m
        if (train_label(i,1) < 0) % bounded support vector (BSV)
            continue;
        end;
        for j = 1:n %check the line connecting example i to all support vectors j
            svind = sv_inds(j,1);
            if (svind == i)
                A(i,i) = 1;
                continue;
            end;
            %if (A(i,svind) == 1)
            if (A(svind,i) == 1)
                continue;
            end;
            X_new = pointsOnLine(X(i,:), X(svind,:), 2);
            [pts_label, accuracy, prob_estimates] = svmpredict(ones(size(X_new,1),1), X_new, model);
            if (size(find(pts_label == -1),1) == 0)
                A(i,svind) = 1;
                A(svind,i) = 1;
            end;
        end;
    end;    
    
% check the line between each example and the support vectors. 
% If for all the points on the line the radius is smaller than the radius of the
% bounding SV sphere then the point is in one cluster with this support vector
function A = buildAdjacency1(X, model, sv_inds, train_label)    
    m = size(X,1);
    n = size(sv_inds,1);
    A = zeros(m);
    B = zeros(n);
    cnt = 0;    
    for i = 1:n
        svind_i = sv_inds(i,1);
        if (train_label(svind_i,1) == -1) % bounded support vector (BSV)
             continue;
        end;
        for j = i:n %check the line connecting example i to all support vectors j
            svind_j = sv_inds(j,1);
            if (svind_j == svind_i)
                A(svind_i,svind_i) = 1;
                B(i, i) = 1;
                continue;
            end;
            if (A(svind_j,svind_i) == 1)
                continue;
            end;
            X_new = pointsOnLine(X(svind_i,:), X(svind_j,:), 20);
            [pts_label, accuracy, prob_estimates] = svmpredict(ones(size(X_new,1),1), X_new, model);
            if (size(find(pts_label == -1),1) == 0)
                A(svind_i,svind_j) = 1;
                A(svind_j,svind_i) = 1;
                B(i, j) = 1;
                B(j, i) = 1;
            end;
        end;
    end;  
   
function X_new = pointsOnLine(x1, x2, numpoints)
    step = (x2 - x1)/(numpoints+1);
    X_new = [];
    for i = 1:numpoints
        X_new = [X_new; x1 + i*step];
    end;    
    
function Ind = getIndices(X, svs)
    Ind = [];
    for i = 1:size(svs,1)
        tmpInd = findInd(X, svs(i,:));
        Ind = [Ind; tmpInd];
    end;
    
function Ind = findInd(X, sv)
    for i = 1:size(X,1)
        isInd = 1;
        for j = 1:size(X,2)
            if (X(i,j) ~= sv(1,j))
                isInd = 0;
                break;
            end;
        end;
        if (isInd == 1)
            Ind = i;
            return;
        end;
    end;
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Input functions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [X, Y] = loadDataXY(file)    
    dims_data = 2;
    dims_file = 3;
    D = loadData(file,dims_file);
    cols= size(D,2);
    X = D(:,1:cols-1);
    Y = D(:,cols);
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% Plotting functions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function plotOrigData(X, Y)    
    inds = find(Y == 1);
    figure;
    plot(X(inds,1), X(inds,2), '.b');
    inds = find(Y==2);
    hold on;
    plot(X(inds,1), X(inds, 2), '.r');   
    title('Test data');
    axis([-4,4,-4,4]);    

function plotClustering(X, model, train_label, Lbls)
    svs = full(model.SVs);
    sv_inds = getIndices(X, svs);
    inds_neg = find(train_label < 0);
    inds_pos = find(train_label > 1);
    %figure;
    %hold on;
%     plot(X(inds_neg,1), X(inds_neg,2), '.k');
     plot(svs(:,1), svs(:,2), 'ok'); 
%    plot3(X(inds_neg,1), X(inds_neg,2), -0.15*ones(size(inds_neg,1),1), '.k');
%    plot3(svs(:,1), svs(:,2), -0.15*ones(size(svs,1),1), 'ok'); 

    lbls_type = unique(Lbls);
    num_lbls = size(lbls_type,1);
    color_map = ['b'; 'r'; 'g'; 'm'; 'y'; 'c']; 
    for i = 1:num_lbls
        inds = find(Lbls(:,1) == lbls_type(i,1));
        if (lbls_type(i,1) == 0) % plot the BSVs
            scol = '.k';
        else 
            col = rem(i-1, size(color_map,1))+1;
            scol = ['.' color_map(col,1)];
        end;
        plot(X(inds,1),X(inds,2), scol, 'LineWidth', 1);        
        %plot3(X(inds,1),X(inds,2), -0.15*ones(size(inds,1),1), ['.' color_map(col,1)], 'LineWidth', 1);
    end;
    %axis([-4,4,-4,4]);    
    
function plotContours(X, model, gamma)
    minx = min(X(:,1));
    maxx = max(X(:,1));
    miny = min(X(:,2));
    maxy = max(X(:,2));
    stepx = (maxx - minx)/50;
    stepy = (maxy - miny)/50;
    Xcoord = minx:stepx:maxx;
    Ycoord = miny:stepy:maxy;
    Z = zeros(size(Ycoord,2), size(Xcoord,2));
    for i = 1:size(Ycoord,2)
        for j = 1:size(Xcoord,2)
            Z(i,j) = model.sv_coef'*KernelG([Xcoord(1,j) Ycoord(1,i)], model, gamma) - model.rho;    
            if (Z(i,j) < -0.05)
                Z(i,j) = -0.05;
            elseif (Z(i,j) > 1)
                Z(i,j) = 1;
            end;
        end;
    end;
    maxZ = max(Z,1);
    maxZ = max(Z,2);
    sliceStep = maxZ / 10;
    slices = 0:sliceStep:maxZ-sliceStep;
    
    
    figure;
    [Xm,Ym] = meshgrid(Xcoord,Ycoord);
    hold on;
    %surface(Xm,Ym,Z,'EdgeColor',[.8 .8 .8],'FaceColor','none')
    [c, h]=contourf(Xm,Ym,Z,slices);
    set(h, 'LineWidth', 1)
    colormap cool;    
    colorbar;
    h = plot(X(:,1),X(:,2), '.', 'MarkerEdgeColor', [.0 .0 1], 'LineWidth', 1);        
    legend('density estimate', 'examples');
    grid off
    set(gca, 'FontSize', 12);
    axis([-16.5 28.4 -31 22.4]);
    %axis tight
    k = 1;
